@@ -4,7 +4,7 @@
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2008 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2024 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <stack>
 #include <wx/filefn.h>
 
 #include <eda_item.h>
@@ -31,18 +32,17 @@
 #include <string_utils.h>
 #include <kiway.h>
 #include <plotters/plotter.h>
+#include <sch_plotter.h>
 #include <project.h>
+#include <project_sch.h>
 #include <reporter.h>
-#include <sch_draw_panel.h>
 #include <trace_helpers.h>
 #include <sch_edit_frame.h>
 #include <sch_item.h>
 
 #include <symbol_library.h>
 #include <connection_graph.h>
-#include <lib_item.h>
 #include <lib_pin.h>
-#include <lib_shape.h>
 #include <sch_symbol.h>
 #include <sch_junction.h>
 #include <sch_line.h>
@@ -54,15 +54,13 @@
 #include <symbol_lib_table.h>
 #include <tool/common_tools.h>
 #include <sim/sim_model.h> // For V6 to V7 simulation model migration.
-#include <sim/sim_value.h> //
 #include <locale_io.h>
 
 #include <algorithm>
 
 // TODO(JE) Debugging only
-#include <profile.h>
+#include <core/profile.h>
 #include "sch_bus_entry.h"
-#include "sim/sim_model_ideal.h"
 
 /*
  * Flag to enable profiling of the TestDanglingEnds() function.
@@ -200,12 +198,20 @@ void SCH_SCREEN::Append( SCH_ITEM* aItem, bool aUpdateLibSymbol )
 
                             wxCHECK2( foundSymbol, continue );
 
+                            wxString tmp = symbol->GetLibSymbolRef()->GetName();
+
+                            // Temporarily update the new symbol library symbol name so it
+                            // doesn't fail on the name comparison below.
+                            symbol->GetLibSymbolRef()->SetName( foundSymbol->GetName() );
+
                             if( *foundSymbol == *symbol->GetLibSymbolRef() )
                             {
                                 newName = libSymbolName;
+                                symbol->GetLibSymbolRef()->SetName( tmp );
                                 break;
                             }
 
+                            symbol->GetLibSymbolRef()->SetName( tmp );
                             foundSymbol = nullptr;
                         }
 
@@ -451,6 +457,11 @@ std::set<SCH_ITEM*> SCH_SCREEN::MarkConnections( SCH_LINE* aSegment, bool aSecon
 
                     break;
                 }
+                else if( line->GetLayer() == LAYER_NOTES && item->GetLayer() == LAYER_NOTES )
+                {
+                    retval.insert( line );
+                    to_search.push( line );
+                }
             }
         }
     }
@@ -491,9 +502,9 @@ bool SCH_SCREEN::IsExplicitJunctionNeeded( const VECTOR2I& aPosition ) const
     return isJunction && !hasBusEntry && !hasExplicitJunction;
 }
 
-TEXT_SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       aPosition,
-                                                         TEXT_SPIN_STYLE       aDefaultOrientation,
-                                                         const SCH_SHEET_PATH* aSheet ) const
+SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       aPosition,
+                                                    SPIN_STYLE            aDefaultOrientation,
+                                                    const SCH_SHEET_PATH* aSheet ) const
 {
     auto ret = aDefaultOrientation;
     for( SCH_ITEM* item : Items().Overlapping( aPosition ) )
@@ -516,18 +527,18 @@ TEXT_SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       a
                     // bus is vertical -> label shall be horizontal and
                     // shall be placed to the side where the bus entry is
                     if( aPosition.x < bus->GetPosition().x )
-                        ret = TEXT_SPIN_STYLE::LEFT;
+                        ret = SPIN_STYLE::LEFT;
                     else if( aPosition.x > bus->GetPosition().x )
-                        ret = TEXT_SPIN_STYLE::RIGHT;
+                        ret = SPIN_STYLE::RIGHT;
                 }
                 else if( bus->Angle().AsDegrees() == 0.0 )
                 {
                     // bus is horizontal -> label shall be vertical and
                     // shall be placed to the side where the bus entry is
                     if( aPosition.y < bus->GetPosition().y )
-                        ret = TEXT_SPIN_STYLE::UP;
+                        ret = SPIN_STYLE::UP;
                     else if( aPosition.y > bus->GetPosition().y )
-                        ret = TEXT_SPIN_STYLE::BOTTOM;
+                        ret = SPIN_STYLE::BOTTOM;
                 }
             }
         }
@@ -542,28 +553,16 @@ TEXT_SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       a
             if( -45 < angle && angle <= 45 )
             {
                 if( line->GetStartPoint().x <= line->GetEndPoint().x )
-                {
-                    ret = line->GetEndPoint() == aPosition ? TEXT_SPIN_STYLE::RIGHT
-                                                           : TEXT_SPIN_STYLE::LEFT;
-                }
+                    ret = line->GetEndPoint() == aPosition ? SPIN_STYLE::RIGHT : SPIN_STYLE::LEFT;
                 else
-                {
-                    ret = line->GetEndPoint() == aPosition ? TEXT_SPIN_STYLE::LEFT
-                                                           : TEXT_SPIN_STYLE::RIGHT;
-                }
+                    ret = line->GetEndPoint() == aPosition ? SPIN_STYLE::LEFT : SPIN_STYLE::RIGHT;
             }
             else
             {
                 if( line->GetStartPoint().y <= line->GetEndPoint().y )
-                {
-                    ret = line->GetEndPoint() == aPosition ? TEXT_SPIN_STYLE::BOTTOM
-                                                           : TEXT_SPIN_STYLE::UP;
-                }
+                    ret = line->GetEndPoint() == aPosition ? SPIN_STYLE::BOTTOM : SPIN_STYLE::UP;
                 else
-                {
-                    ret = line->GetEndPoint() == aPosition ? TEXT_SPIN_STYLE::UP
-                                                           : TEXT_SPIN_STYLE::BOTTOM;
-                }
+                    ret = line->GetEndPoint() == aPosition ? SPIN_STYLE::UP : SPIN_STYLE::BOTTOM;
             }
         }
         break;
@@ -576,104 +575,96 @@ TEXT_SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       a
             {
                 if( pin->GetPosition() == aPosition )
                 {
-                    if( pin->GetOrientation() == PIN_RIGHT )
-                    {
-                        ret = TEXT_SPIN_STYLE::LEFT;
-                    }
-                    else if( pin->GetOrientation() == PIN_LEFT )
-                    {
-                        ret = TEXT_SPIN_STYLE::RIGHT;
-                    }
-                    else if( pin->GetOrientation() == PIN_UP )
-                    {
-                        ret = TEXT_SPIN_STYLE::BOTTOM;
-                    }
-                    else if( pin->GetOrientation() == PIN_DOWN )
-                    {
-                        ret = TEXT_SPIN_STYLE::UP;
-                    }
+                    if( pin->GetOrientation() == PIN_ORIENTATION::PIN_RIGHT )
+                        ret = SPIN_STYLE::LEFT;
+                    else if( pin->GetOrientation() == PIN_ORIENTATION::PIN_LEFT )
+                        ret = SPIN_STYLE::RIGHT;
+                    else if( pin->GetOrientation() == PIN_ORIENTATION::PIN_UP )
+                        ret = SPIN_STYLE::BOTTOM;
+                    else if( pin->GetOrientation() == PIN_ORIENTATION::PIN_DOWN )
+                        ret = SPIN_STYLE::UP;
 
                     switch( static_cast<SYMBOL_ORIENTATION_T>(
                             symbol->GetOrientation() & ( ~( SYM_MIRROR_X | SYM_MIRROR_Y ) ) ) )
                     {
                     case SYM_ROTATE_CLOCKWISE:
                     case SYM_ORIENT_90:
-                        if( ret == TEXT_SPIN_STYLE::UP )
-                            ret = TEXT_SPIN_STYLE::LEFT;
-                        else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                            ret = TEXT_SPIN_STYLE::RIGHT;
-                        else if( ret == TEXT_SPIN_STYLE::LEFT )
-                            ret = TEXT_SPIN_STYLE::BOTTOM;
-                        else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                            ret = TEXT_SPIN_STYLE::UP;
+                        if( ret == SPIN_STYLE::UP )
+                            ret = SPIN_STYLE::LEFT;
+                        else if( ret == SPIN_STYLE::BOTTOM )
+                            ret = SPIN_STYLE::RIGHT;
+                        else if( ret == SPIN_STYLE::LEFT )
+                            ret = SPIN_STYLE::BOTTOM;
+                        else if( ret == SPIN_STYLE::RIGHT )
+                            ret = SPIN_STYLE::UP;
 
                         if( symbol->GetOrientation() & SYM_MIRROR_X )
                         {
-                            if( ret == TEXT_SPIN_STYLE::UP )
-                                ret = TEXT_SPIN_STYLE::BOTTOM;
-                            else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                                ret = TEXT_SPIN_STYLE::UP;
+                            if( ret == SPIN_STYLE::UP )
+                                ret = SPIN_STYLE::BOTTOM;
+                            else if( ret == SPIN_STYLE::BOTTOM )
+                                ret = SPIN_STYLE::UP;
                         }
 
                         if( symbol->GetOrientation() & SYM_MIRROR_Y )
                         {
-                            if( ret == TEXT_SPIN_STYLE::LEFT )
-                                ret = TEXT_SPIN_STYLE::RIGHT;
-                            else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                                ret = TEXT_SPIN_STYLE::LEFT;
+                            if( ret == SPIN_STYLE::LEFT )
+                                ret = SPIN_STYLE::RIGHT;
+                            else if( ret == SPIN_STYLE::RIGHT )
+                                ret = SPIN_STYLE::LEFT;
                         }
                         break;
                     case SYM_ROTATE_COUNTERCLOCKWISE:
                     case SYM_ORIENT_270:
-                        if( ret == TEXT_SPIN_STYLE::UP )
-                            ret = TEXT_SPIN_STYLE::RIGHT;
-                        else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                            ret = TEXT_SPIN_STYLE::LEFT;
-                        else if( ret == TEXT_SPIN_STYLE::LEFT )
-                            ret = TEXT_SPIN_STYLE::UP;
-                        else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                            ret = TEXT_SPIN_STYLE::BOTTOM;
+                        if( ret == SPIN_STYLE::UP )
+                            ret = SPIN_STYLE::RIGHT;
+                        else if( ret == SPIN_STYLE::BOTTOM )
+                            ret = SPIN_STYLE::LEFT;
+                        else if( ret == SPIN_STYLE::LEFT )
+                            ret = SPIN_STYLE::UP;
+                        else if( ret == SPIN_STYLE::RIGHT )
+                            ret = SPIN_STYLE::BOTTOM;
 
                         if( symbol->GetOrientation() & SYM_MIRROR_X )
                         {
-                            if( ret == TEXT_SPIN_STYLE::UP )
-                                ret = TEXT_SPIN_STYLE::BOTTOM;
-                            else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                                ret = TEXT_SPIN_STYLE::UP;
+                            if( ret == SPIN_STYLE::UP )
+                                ret = SPIN_STYLE::BOTTOM;
+                            else if( ret == SPIN_STYLE::BOTTOM )
+                                ret = SPIN_STYLE::UP;
                         }
 
                         if( symbol->GetOrientation() & SYM_MIRROR_Y )
                         {
-                            if( ret == TEXT_SPIN_STYLE::LEFT )
-                                ret = TEXT_SPIN_STYLE::RIGHT;
-                            else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                                ret = TEXT_SPIN_STYLE::LEFT;
+                            if( ret == SPIN_STYLE::LEFT )
+                                ret = SPIN_STYLE::RIGHT;
+                            else if( ret == SPIN_STYLE::RIGHT )
+                                ret = SPIN_STYLE::LEFT;
                         }
                         break;
                     case SYM_ORIENT_180:
-                        if( ret == TEXT_SPIN_STYLE::UP )
-                            ret = TEXT_SPIN_STYLE::BOTTOM;
-                        else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                            ret = TEXT_SPIN_STYLE::UP;
-                        else if( ret == TEXT_SPIN_STYLE::LEFT )
-                            ret = TEXT_SPIN_STYLE::RIGHT;
-                        else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                            ret = TEXT_SPIN_STYLE::LEFT;
+                        if( ret == SPIN_STYLE::UP )
+                            ret = SPIN_STYLE::BOTTOM;
+                        else if( ret == SPIN_STYLE::BOTTOM )
+                            ret = SPIN_STYLE::UP;
+                        else if( ret == SPIN_STYLE::LEFT )
+                            ret = SPIN_STYLE::RIGHT;
+                        else if( ret == SPIN_STYLE::RIGHT )
+                            ret = SPIN_STYLE::LEFT;
 
                         if( symbol->GetOrientation() & SYM_MIRROR_X )
                         {
-                            if( ret == TEXT_SPIN_STYLE::UP )
-                                ret = TEXT_SPIN_STYLE::BOTTOM;
-                            else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                                ret = TEXT_SPIN_STYLE::UP;
+                            if( ret == SPIN_STYLE::UP )
+                                ret = SPIN_STYLE::BOTTOM;
+                            else if( ret == SPIN_STYLE::BOTTOM )
+                                ret = SPIN_STYLE::UP;
                         }
 
                         if( symbol->GetOrientation() & SYM_MIRROR_Y )
                         {
-                            if( ret == TEXT_SPIN_STYLE::LEFT )
-                                ret = TEXT_SPIN_STYLE::RIGHT;
-                            else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                                ret = TEXT_SPIN_STYLE::LEFT;
+                            if( ret == SPIN_STYLE::LEFT )
+                                ret = SPIN_STYLE::RIGHT;
+                            else if( ret == SPIN_STYLE::RIGHT )
+                                ret = SPIN_STYLE::LEFT;
                         }
                         break;
                     case SYM_ORIENT_0:
@@ -681,18 +672,18 @@ TEXT_SPIN_STYLE SCH_SCREEN::GetLabelOrientationForPoint( const VECTOR2I&       a
                     default:
                         if( symbol->GetOrientation() & SYM_MIRROR_X )
                         {
-                            if( ret == TEXT_SPIN_STYLE::UP )
-                                ret = TEXT_SPIN_STYLE::BOTTOM;
-                            else if( ret == TEXT_SPIN_STYLE::BOTTOM )
-                                ret = TEXT_SPIN_STYLE::UP;
+                            if( ret == SPIN_STYLE::UP )
+                                ret = SPIN_STYLE::BOTTOM;
+                            else if( ret == SPIN_STYLE::BOTTOM )
+                                ret = SPIN_STYLE::UP;
                         }
 
                         if( symbol->GetOrientation() & SYM_MIRROR_Y )
                         {
-                            if( ret == TEXT_SPIN_STYLE::LEFT )
-                                ret = TEXT_SPIN_STYLE::RIGHT;
-                            else if( ret == TEXT_SPIN_STYLE::RIGHT )
-                                ret = TEXT_SPIN_STYLE::LEFT;
+                            if( ret == SPIN_STYLE::LEFT )
+                                ret = SPIN_STYLE::RIGHT;
+                            else if( ret == SPIN_STYLE::RIGHT )
+                                ret = SPIN_STYLE::LEFT;
                         }
                         break;
                     }
@@ -893,10 +884,10 @@ void SCH_SCREEN::UpdateSymbolLinks( REPORTER* aReporter )
     wxString msg;
     std::unique_ptr< LIB_SYMBOL > libSymbol;
     std::vector<SCH_SYMBOL*> symbols;
-    SYMBOL_LIB_TABLE* libs = Schematic()->Prj().SchSymbolLibTable();
+    SYMBOL_LIB_TABLE* libs = PROJECT_SCH::SchSymbolLibTable( &Schematic()->Prj() );
 
     // This will be a nullptr if an s-expression schematic is loaded.
-    SYMBOL_LIBS* legacyLibs = Schematic()->Prj().SchLibs();
+    SYMBOL_LIBS* legacyLibs = PROJECT_SCH::SchLibs( &Schematic()->Prj() );
 
     for( SCH_ITEM* item : Items().OfType( SCH_SYMBOL_T ) )
         symbols.push_back( static_cast<SCH_SYMBOL*>( item ) );
@@ -1118,7 +1109,7 @@ void SCH_SCREEN::Print( const RENDER_SETTINGS* aSettings )
 }
 
 
-void SCH_SCREEN::Plot( PLOTTER* aPlotter ) const
+void SCH_SCREEN::Plot( PLOTTER* aPlotter, const SCH_PLOT_SETTINGS& aPlotSettings ) const
 {
     // Ensure links are up to date, even if a library was reloaded for some reason:
     std::vector<SCH_ITEM*>   junctions;
@@ -1173,21 +1164,21 @@ void SCH_SCREEN::Plot( PLOTTER* aPlotter ) const
     for( const SCH_ITEM* item : bitmaps )
     {
         aPlotter->SetCurrentLineWidth( std::max( item->GetPenWidth(), defaultPenWidth ) );
-        item->Plot( aPlotter, background );
+        item->Plot( aPlotter, background, aPlotSettings );
     }
 
     // Plot the background items
     for( const SCH_ITEM* item : other )
     {
         aPlotter->SetCurrentLineWidth( std::max( item->GetPenWidth(), defaultPenWidth ) );
-        item->Plot( aPlotter, background );
+        item->Plot( aPlotter, background, aPlotSettings );
     }
 
     // Plot the foreground items
     for( const SCH_ITEM* item : other )
     {
         aPlotter->SetCurrentLineWidth( std::max( item->GetPenWidth(), defaultPenWidth ) );
-        item->Plot( aPlotter, !background );
+        item->Plot( aPlotter, !background, aPlotSettings );
     }
 
     // After plotting the symbols as a group above (in `other`), we need to overplot the pins
@@ -1199,7 +1190,7 @@ void SCH_SCREEN::Plot( PLOTTER* aPlotter ) const
         for( SCH_FIELD field : sym->GetFields() )
         {
             field.ClearRenderCache();
-            field.Plot( aPlotter, false );
+            field.Plot( aPlotter, false, aPlotSettings );
         }
 
         sym->PlotPins( aPlotter );
@@ -1211,7 +1202,7 @@ void SCH_SCREEN::Plot( PLOTTER* aPlotter ) const
     for( const SCH_ITEM* item : junctions )
     {
         aPlotter->SetCurrentLineWidth( std::max( item->GetPenWidth(), defaultPenWidth ) );
-        item->Plot( aPlotter, !background );
+        item->Plot( aPlotter, !background, aPlotSettings );
     }
 }
 
@@ -1596,7 +1587,7 @@ void SCH_SCREEN::SetLegacySymbolInstanceData()
         SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
 
         // Add missing value and footprint instance data for legacy schematics.
-        for( const SCH_SYMBOL_INSTANCE& instance : symbol->GetInstanceReferences() )
+        for( const SCH_SYMBOL_INSTANCE& instance : symbol->GetInstances() )
         {
             symbol->AddHierarchicalReference( instance.m_Path, instance.m_Reference,
                                               instance.m_Unit );
@@ -1605,6 +1596,26 @@ void SCH_SCREEN::SetLegacySymbolInstanceData()
 }
 
 
+void SCH_SCREEN::FixLegacyPowerSymbolMismatches()
+{
+    for( SCH_ITEM* item : Items().OfType( SCH_SYMBOL_T ) )
+    {
+        SCH_SYMBOL* symbol = static_cast<SCH_SYMBOL*>( item );
+
+        // Fix pre-8.0 legacy power symbols with invisible pins
+        // that have mismatched pin names and value fields
+        if( symbol->GetLibSymbolRef()
+            && symbol->GetLibSymbolRef()->IsPower()
+            && symbol->GetAllLibPins().size() > 0
+            && symbol->GetAllLibPins()[0]->IsGlobalPower()
+            && !symbol->GetAllLibPins()[0]->IsVisible() )
+        {
+            symbol->SetValueFieldText( symbol->GetAllLibPins()[0]->GetName() );
+        }
+    }
+}
+
+
 size_t SCH_SCREEN::getLibSymbolNameMatches( const SCH_SYMBOL& aSymbol,
                                             std::vector<wxString>& aMatches )
 {
@@ -1618,7 +1629,7 @@ size_t SCH_SCREEN::getLibSymbolNameMatches( const SCH_SYMBOL& aSymbol,
     long tmp;
     wxString suffix;
 
-    for( auto pair : m_libSymbols )
+    for( auto& pair : m_libSymbols )
     {
         if( pair.first.StartsWith( searchName, &suffix ) && suffix.ToLong( &tmp ) )
             aMatches.emplace_back( pair.first );
@@ -1634,8 +1645,10 @@ void SCH_SCREEN::PruneOrphanedSymbolInstances( const wxString& aProjectName,
     // The project name cannot be empty.  Projects older than 7.0 did not save project names
     // when saving instance data.  Running this algorithm with an empty project name would
     // clobber all instance data for projects other than the current one when a schematic
-    // file is shared across multiple projects.
-    wxCHECK( !aProjectName.IsEmpty(), /* void */ );
+    // file is shared across multiple projects.  Because running the schematic editor in
+    // stand alone mode can result in an empty project name, do not assert here.
+    if( aProjectName.IsEmpty() )
+        return;
 
     for( SCH_ITEM* item : Items().OfType( SCH_SYMBOL_T ) )
     {
@@ -1644,7 +1657,7 @@ void SCH_SCREEN::PruneOrphanedSymbolInstances( const wxString& aProjectName,
         wxCHECK2( symbol, continue );
 
         std::set<KIID_PATH> pathsToPrune;
-        const std::vector<SCH_SYMBOL_INSTANCE> instances = symbol->GetInstanceReferences();
+        const std::vector<SCH_SYMBOL_INSTANCE> instances = symbol->GetInstances();
 
         for( const SCH_SYMBOL_INSTANCE& instance : instances )
         {
@@ -1655,7 +1668,7 @@ void SCH_SCREEN::PruneOrphanedSymbolInstances( const wxString& aProjectName,
             std::optional<SCH_SHEET_PATH> pathFound =
                     aValidSheetPaths.GetSheetPathByKIIDPath( instance.m_Path );
 
-            // Check for paths that do not exist in the current  project and paths that do
+            // Check for paths that do not exist in the current project and paths that do
             // not contain the current symbol.
             if( !pathFound )
                 pathsToPrune.emplace( instance.m_Path );
@@ -1673,6 +1686,53 @@ void SCH_SCREEN::PruneOrphanedSymbolInstances( const wxString& aProjectName,
 }
 
 
+void SCH_SCREEN::PruneOrphanedSheetInstances( const wxString& aProjectName,
+                                              const SCH_SHEET_LIST& aValidSheetPaths )
+{
+    // The project name cannot be empty.  Projects older than 7.0 did not save project names
+    // when saving instance data.  Running this algorithm with an empty project name would
+    // clobber all instance data for projects other than the current one when a schematic
+    // file is shared across multiple projects.  Because running the schematic editor in
+    // stand alone mode can result in an empty project name, do not assert here.
+    if( aProjectName.IsEmpty() )
+        return;
+
+    for( SCH_ITEM* item : Items().OfType( SCH_SHEET_T ) )
+    {
+        SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
+
+        wxCHECK2( sheet, continue );
+
+        std::set<KIID_PATH> pathsToPrune;
+        const std::vector<SCH_SHEET_INSTANCE> instances = sheet->GetInstances();
+
+        for( const SCH_SHEET_INSTANCE& instance : instances )
+        {
+            // Ignore instance paths from other projects.
+            if( aProjectName != instance.m_ProjectName )
+                continue;
+
+            std::optional<SCH_SHEET_PATH> pathFound =
+                    aValidSheetPaths.GetSheetPathByKIIDPath( instance.m_Path );
+
+            // Check for paths that do not exist in the current project and paths that do
+            // not contain the current symbol.
+            if( !pathFound )
+                pathsToPrune.emplace( instance.m_Path );
+            else if( pathFound.value().LastScreen() != this )
+                pathsToPrune.emplace( pathFound.value().Path() );
+        }
+
+        for( const KIID_PATH& sheetPath : pathsToPrune )
+        {
+            wxLogTrace( traceSchSheetPaths, wxS( "Pruning project '%s' sheet instance %s." ),
+                        aProjectName, sheetPath.AsString() );
+            sheet->RemoveInstance( sheetPath );
+        }
+    }
+}
+
+
 #if defined(DEBUG)
 void SCH_SCREEN::Show( int nestLevel, std::ostream& os ) const
 {
@@ -1879,7 +1939,7 @@ void SCH_SCREENS::DeleteMarker( SCH_MARKER* aMarker )
 }
 
 
-void SCH_SCREENS::DeleteMarkers( enum MARKER_BASE::TYPEMARKER aMarkerType, int aErrorCode,
+void SCH_SCREENS::DeleteMarkers( enum MARKER_BASE::MARKER_T aMarkerType, int aErrorCode,
                                  bool aIncludeExclusions )
 {
     for( SCH_SCREEN* screen = GetFirst(); screen; screen = GetNext() )
@@ -1905,7 +1965,7 @@ void SCH_SCREENS::DeleteMarkers( enum MARKER_BASE::TYPEMARKER aMarkerType, int a
 }
 
 
-void SCH_SCREENS::DeleteAllMarkers( enum MARKER_BASE::TYPEMARKER aMarkerType,
+void SCH_SCREENS::DeleteAllMarkers( enum MARKER_BASE::MARKER_T aMarkerType,
                                     bool aIncludeExclusions )
 {
     DeleteMarkers( aMarkerType, ERCE_UNSPECIFIED, aIncludeExclusions );
@@ -2076,6 +2136,13 @@ void SCH_SCREENS::SetLegacySymbolInstanceData()
 }
 
 
+void SCH_SCREENS::FixLegacyPowerSymbolMismatches()
+{
+    for( SCH_SCREEN* screen = GetFirst(); screen; screen = GetNext() )
+        screen->FixLegacyPowerSymbolMismatches();
+}
+
+
 void SCH_SCREEN::MigrateSimModels()
 {
     LOCALE_IO toggle;
@@ -2092,11 +2159,22 @@ void SCH_SCREEN::MigrateSimModels()
 
 
 void SCH_SCREENS::PruneOrphanedSymbolInstances( const wxString& aProjectName,
-
                                                 const SCH_SHEET_LIST& aValidSheetPaths )
 {
-    wxCHECK( !aProjectName.IsEmpty(), /* void */ );
+    if( aProjectName.IsEmpty() )
+        return;
 
     for( SCH_SCREEN* screen = GetFirst(); screen; screen = GetNext() )
         screen->PruneOrphanedSymbolInstances( aProjectName, aValidSheetPaths );
 }
+
+
+void SCH_SCREENS::PruneOrphanedSheetInstances( const wxString& aProjectName,
+                                               const SCH_SHEET_LIST& aValidSheetPaths )
+{
+    if( aProjectName.IsEmpty() )
+        return;
+
+    for( SCH_SCREEN* screen = GetFirst(); screen; screen = GetNext() )
+        screen->PruneOrphanedSheetInstances( aProjectName, aValidSheetPaths );
+}
