@@ -32,6 +32,7 @@
 #include <eeschema_id.h>
 #include <eeschema_settings.h>
 #include <env_paths.h>
+#include <gal/graphics_abstraction_layer.h>
 #include <kiface_base.h>
 #include <kiplatform/app.h>
 #include <kiway_express.h>
@@ -41,6 +42,7 @@
 #include <symbol_editor_settings.h>
 #include <paths.h>
 #include <pgm_base.h>
+#include <project_sch.h>
 #include <sch_painter.h>
 #include <sch_view.h>
 #include <settings/settings_manager.h>
@@ -51,6 +53,7 @@
 #include <tool/common_tools.h>
 #include <tool/editor_conditions.h>
 #include <tool/picker_tool.h>
+#include <tool/properties_tool.h>
 #include <tool/selection.h>
 #include <tool/tool_dispatcher.h>
 #include <tool/tool_manager.h>
@@ -68,7 +71,9 @@
 #include <widgets/wx_infobar.h>
 #include <widgets/lib_tree.h>
 #include <widgets/wx_progress_reporters.h>
+#include <widgets/sch_properties_panel.h>
 #include <widgets/symbol_tree_pane.h>
+#include <widgets/wx_aui_utils.h>
 #include <wildcards_and_files_ext.h>
 #include <panel_sym_lib_table.h>
 #include <string_utils.h>
@@ -78,14 +83,11 @@ bool SYMBOL_EDIT_FRAME::m_showDeMorgan = false;
 
 
 BEGIN_EVENT_TABLE( SYMBOL_EDIT_FRAME, SCH_BASE_FRAME )
-    EVT_SIZE( SYMBOL_EDIT_FRAME::OnSize )
-
     EVT_COMBOBOX( ID_LIBEDIT_SELECT_UNIT_NUMBER, SYMBOL_EDIT_FRAME::OnSelectUnit )
 
     // menubar commands
     EVT_MENU( wxID_EXIT, SYMBOL_EDIT_FRAME::OnExitKiCad )
     EVT_MENU( wxID_CLOSE, SYMBOL_EDIT_FRAME::CloseWindow )
-    EVT_MENU( ID_GRID_SETTINGS, SCH_BASE_FRAME::OnGridSettings )
 
     // Update user interface elements.
     EVT_UPDATE_UI( ID_LIBEDIT_SELECT_UNIT_NUMBER, SYMBOL_EDIT_FRAME::OnUpdateUnitNumber )
@@ -110,13 +112,17 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_treePane = nullptr;
     m_libMgr = nullptr;
     m_unit = 1;
-    m_convert = 1;
+    m_bodyStyle = 1;
     m_aboutTitle = _HKI( "KiCad Symbol Editor" );
 
     wxIcon icon;
     wxIconBundle icon_bundle;
 
-    icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_libedit ) );
+    icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_libedit, 48 ) );
+    icon_bundle.AddIcon( icon );
+    icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_libedit, 256 ) );
+    icon_bundle.AddIcon( icon );
+    icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_libedit, 128 ) );
     icon_bundle.AddIcon( icon );
     icon.CopyFromBitmap( KiBitmap( BITMAPS::icon_libedit_32 ) );
     icon_bundle.AddIcon( icon );
@@ -144,6 +150,7 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     SyncLibraries( false, loadingCancelled );
     m_treePane = new SYMBOL_TREE_PANE( this, m_libMgr );
+    m_treePane->GetLibTree()->SetSortMode( (LIB_TREE_MODEL_ADAPTER::SORT_MODE) m_settings->m_LibrarySortMode );
 
     resolveCanvasType();
     SwitchCanvas( m_canvasType );
@@ -174,6 +181,9 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     UpdateSymbolMsgPanelInfo();
     RebuildSymbolUnitsList();
 
+    m_propertiesPanel = new SCH_PROPERTIES_PANEL( this, this );
+    m_propertiesPanel->SetSplitterProportion( m_settings->m_AuiPanels.properties_splitter );
+
     m_auimgr.SetManagedWindow( this );
 
     CreateInfoBar();
@@ -188,8 +198,15 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     // Columns; layers 1 - 3
     m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( "SymbolTree" )
                       .Left().Layer( 3 )
+                      .TopDockable( false ).BottomDockable( false )
                       .Caption( _( "Libraries" ) )
-                      .MinSize( 250, -1 ).BestSize( 250, -1 ) );
+                      .MinSize( FromDIP( 250 ), -1 ).BestSize( FromDIP( 250 ), -1 ) );
+
+    m_auimgr.AddPane( m_propertiesPanel, defaultPropertiesPaneInfo( this ) );
+    // Show or hide m_propertiesPanel depending on current settings:
+    wxAuiPaneInfo& propertiesPaneInfo = m_auimgr.GetPane( PropertiesPaneName() );
+    propertiesPaneInfo.Show( m_settings->m_AuiPanels.show_properties );
+
     m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
                       .Left().Layer( 2 ) );
 
@@ -203,23 +220,7 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     FinishAUIInitialization();
 
     if( m_settings->m_LibWidth > 0 )
-    {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "SymbolTree" );
-
-        // wxAUI hack: force width by setting MinSize() and then Fixed()
-        // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
-        treePane.MinSize( m_settings->m_LibWidth, -1 );
-        treePane.Fixed();
-        m_auimgr.Update();
-
-        // now make it resizable again
-        treePane.Resizable();
-        m_auimgr.Update();
-
-        // Note: DO NOT call m_auimgr.Update() anywhere after this; it will nuke the size
-        // back to minimum.
-        treePane.MinSize( 250, -1 );
-    }
+        SetAuiPaneSize( m_auimgr, m_auimgr.GetPane( "SymbolTree" ), m_settings->m_LibWidth, -1 );
 
     Raise();
     Show( true );
@@ -239,9 +240,9 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     bbox.SetSize( max_size_x, max_size_y );
     GetCanvas()->GetView()->SetBoundary( bbox );
 
-    m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+    m_toolManager->RunAction( ACTIONS::zoomFitScreen );
 
-    m_acceptedExts.emplace( KiCadSymbolLibFileExtension, &ACTIONS::ddAddLibrary );
+    m_acceptedExts.emplace( FILEEXT::KiCadSymbolLibFileExtension, &ACTIONS::ddAddLibrary );
     DragAcceptFiles( true );
 
     KIPLATFORM::APP::SetShutdownBlockReason( this, _( "Library changes are unsaved" ) );
@@ -265,6 +266,8 @@ SYMBOL_EDIT_FRAME::~SYMBOL_EDIT_FRAME()
     if( m_toolManager )
         m_toolManager->ShutdownAllTools();
 
+    setSymWatcher( nullptr );
+
     if( IsSymbolFromSchematic() )
     {
         delete m_symbol;
@@ -278,8 +281,21 @@ SYMBOL_EDIT_FRAME::~SYMBOL_EDIT_FRAME()
     // current screen is destroyed in EDA_DRAW_FRAME
     SetScreen( m_dummyScreen );
 
-    auto libedit = Pgm().GetSettingsManager().GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
-    Pgm().GetSettingsManager().Save( libedit );
+    SYMBOL_EDITOR_SETTINGS* cfg = nullptr;
+
+    try
+    {
+        cfg = Pgm().GetSettingsManager().GetAppSettings<SYMBOL_EDITOR_SETTINGS>();
+    }
+    catch( const std::runtime_error& e )
+    {
+        wxFAIL_MSG( e.what() );
+    }
+
+    if( cfg )
+    {
+        Pgm().GetSettingsManager().Save( cfg );
+    }
 
     delete m_libMgr;
 }
@@ -306,6 +322,12 @@ void SYMBOL_EDIT_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 
     m_settings->m_ShowPinElectricalType  = GetRenderSettings()->m_ShowPinsElectricalType;
     m_settings->m_LibWidth               = m_treePane->GetSize().x;
+
+    m_settings->m_LibrarySortMode = m_treePane->GetLibTree()->GetSortMode();
+
+    m_settings->m_AuiPanels.properties_splitter = m_propertiesPanel->SplitterProportion();
+    bool prop_shown = m_auimgr.GetPane( PropertiesPaneName() ).IsShown();
+    m_settings->m_AuiPanels.show_properties = prop_shown;
 }
 
 
@@ -348,6 +370,7 @@ void SYMBOL_EDIT_FRAME::setupTools()
     m_toolManager->RegisterTool( new SYMBOL_EDITOR_MOVE_TOOL );
     m_toolManager->RegisterTool( new SYMBOL_EDITOR_EDIT_TOOL );
     m_toolManager->RegisterTool( new SYMBOL_EDITOR_CONTROL );
+    m_toolManager->RegisterTool( new PROPERTIES_TOOL );
     m_toolManager->InitTools();
 
     // Run the selection tool, it is supposed to be always active
@@ -386,8 +409,8 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
     auto symbolModifiedCondition =
             [this]( const SELECTION& sel )
             {
-                return m_libMgr->IsSymbolModified( GetTargetLibId().GetLibItemName(),
-                                                   GetTargetLibId().GetLibNickname() );
+                return m_libMgr && m_libMgr->IsSymbolModified( GetTargetLibId().GetLibItemName(),
+                                                               GetTargetLibId().GetLibNickname() );
             };
 
     auto libSelectedCondition =
@@ -408,42 +431,44 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
                 return getTargetSymbol() != nullptr;
             };
 
-    mgr->SetConditions( ACTIONS::saveAll,           ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( ACTIONS::save,              ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( EE_ACTIONS::saveLibraryAs,  ENABLE( libSelectedCondition ) );
-    mgr->SetConditions( EE_ACTIONS::saveSymbolAs,   ENABLE( saveSymbolAsCondition ) );
-    mgr->SetConditions( EE_ACTIONS::newSymbol,      ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( EE_ACTIONS::importSymbol,   ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( ACTIONS::saveAll,             ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( ACTIONS::save,                ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( EE_ACTIONS::saveLibraryAs,    ENABLE( libSelectedCondition ) );
+    mgr->SetConditions( EE_ACTIONS::saveSymbolCopyAs, ENABLE( saveSymbolAsCondition ) );
+    mgr->SetConditions( EE_ACTIONS::newSymbol,        ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( EE_ACTIONS::importSymbol,     ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
 
-    mgr->SetConditions( ACTIONS::undo,              ENABLE( haveSymbolCond && cond.UndoAvailable() ) );
-    mgr->SetConditions( ACTIONS::redo,              ENABLE( haveSymbolCond && cond.RedoAvailable() ) );
-    mgr->SetConditions( ACTIONS::revert,            ENABLE( symbolModifiedCondition ) );
+    mgr->SetConditions( ACTIONS::undo,                ENABLE( haveSymbolCond && cond.UndoAvailable() ) );
+    mgr->SetConditions( ACTIONS::redo,                ENABLE( haveSymbolCond && cond.RedoAvailable() ) );
+    mgr->SetConditions( ACTIONS::revert,              ENABLE( symbolModifiedCondition ) );
 
-    mgr->SetConditions( ACTIONS::toggleGrid,        CHECK( cond.GridVisible() ) );
-    mgr->SetConditions( ACTIONS::toggleCursorStyle, CHECK( cond.FullscreenCursor() ) );
-    mgr->SetConditions( ACTIONS::millimetersUnits,  CHECK( cond.Units( EDA_UNITS::MILLIMETRES ) ) );
-    mgr->SetConditions( ACTIONS::inchesUnits,       CHECK( cond.Units( EDA_UNITS::INCHES ) ) );
-    mgr->SetConditions( ACTIONS::milsUnits,         CHECK( cond.Units( EDA_UNITS::MILS ) ) );
+    mgr->SetConditions( ACTIONS::toggleGrid,          CHECK( cond.GridVisible() ) );
+    mgr->SetConditions( ACTIONS::toggleGridOverrides, CHECK( cond.GridOverrides() ) );
+    mgr->SetConditions( ACTIONS::toggleCursorStyle,   CHECK( cond.FullscreenCursor() ) );
+    mgr->SetConditions( ACTIONS::millimetersUnits,    CHECK( cond.Units( EDA_UNITS::MILLIMETRES ) ) );
+    mgr->SetConditions( ACTIONS::inchesUnits,         CHECK( cond.Units( EDA_UNITS::INCHES ) ) );
+    mgr->SetConditions( ACTIONS::milsUnits,           CHECK( cond.Units( EDA_UNITS::MILS ) ) );
 
-    mgr->SetConditions( ACTIONS::cut,               ENABLE( isEditableCond ) );
-    mgr->SetConditions( ACTIONS::copy,              ENABLE( haveSymbolCond ) );
-    mgr->SetConditions( ACTIONS::paste,             ENABLE( isEditableCond && SELECTION_CONDITIONS::Idle ) );
-    mgr->SetConditions( ACTIONS::doDelete,          ENABLE( isEditableCond ) );
-    mgr->SetConditions( ACTIONS::duplicate,         ENABLE( isEditableCond ) );
-    mgr->SetConditions( ACTIONS::selectAll,         ENABLE( haveSymbolCond ) );
+    mgr->SetConditions( ACTIONS::cut,                 ENABLE( isEditableCond ) );
+    mgr->SetConditions( ACTIONS::copy,                ENABLE( haveSymbolCond ) );
+    mgr->SetConditions( ACTIONS::paste,               ENABLE( isEditableCond && SELECTION_CONDITIONS::Idle ) );
+    mgr->SetConditions( ACTIONS::doDelete,            ENABLE( isEditableCond ) );
+    mgr->SetConditions( ACTIONS::duplicate,           ENABLE( isEditableCond ) );
+    mgr->SetConditions( ACTIONS::selectAll,           ENABLE( haveSymbolCond ) );
+    mgr->SetConditions( ACTIONS::unselectAll,         ENABLE( haveSymbolCond ) );
 
-    mgr->SetConditions( EE_ACTIONS::rotateCW,       ENABLE( isEditableCond ) );
-    mgr->SetConditions( EE_ACTIONS::rotateCCW,      ENABLE( isEditableCond ) );
-    mgr->SetConditions( EE_ACTIONS::mirrorH,        ENABLE( isEditableCond ) );
-    mgr->SetConditions( EE_ACTIONS::mirrorV,        ENABLE( isEditableCond ) );
+    mgr->SetConditions( EE_ACTIONS::rotateCW,         ENABLE( isEditableCond ) );
+    mgr->SetConditions( EE_ACTIONS::rotateCCW,        ENABLE( isEditableCond ) );
+    mgr->SetConditions( EE_ACTIONS::mirrorH,          ENABLE( isEditableCond ) );
+    mgr->SetConditions( EE_ACTIONS::mirrorV,          ENABLE( isEditableCond ) );
 
-    mgr->SetConditions( ACTIONS::zoomTool,          CHECK( cond.CurrentTool( ACTIONS::zoomTool ) ) );
-    mgr->SetConditions( ACTIONS::selectionTool,     CHECK( cond.CurrentTool( ACTIONS::selectionTool ) ) );
+    mgr->SetConditions( ACTIONS::zoomTool,            CHECK( cond.CurrentTool( ACTIONS::zoomTool ) ) );
+    mgr->SetConditions( ACTIONS::selectionTool,       CHECK( cond.CurrentTool( ACTIONS::selectionTool ) ) );
 
     auto pinTypeCond =
             [this]( const SELECTION& )
             {
-                return GetRenderSettings()->m_ShowPinsElectricalType;
+                return GetRenderSettings() && GetRenderSettings()->m_ShowPinsElectricalType;
             };
 
     auto showCompTreeCond =
@@ -452,9 +477,16 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
                 return IsSymbolTreeShown();
             };
 
+    auto propertiesCond =
+            [this] ( const SELECTION& )
+            {
+                return m_auimgr.GetPane( PropertiesPaneName() ).IsShown();
+            };
+
     mgr->SetConditions( EE_ACTIONS::showElectricalTypes, CHECK( pinTypeCond ) );
     mgr->SetConditions( ACTIONS::toggleBoundingBoxes,    CHECK( cond.BoundingBoxes() ) );
     mgr->SetConditions( EE_ACTIONS::showSymbolTree,      CHECK( showCompTreeCond ) );
+    mgr->SetConditions( ACTIONS::showProperties,         CHECK( propertiesCond ) );
 
     auto demorganCond =
             [this]( const SELECTION& )
@@ -465,13 +497,13 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
     auto demorganStandardCond =
             [this]( const SELECTION& )
             {
-                return m_convert == LIB_ITEM::LIB_CONVERT::BASE;
+                return m_bodyStyle == LIB_ITEM::BODY_STYLE::BASE;
             };
 
     auto demorganAlternateCond =
             [this]( const SELECTION& )
             {
-                return m_convert == LIB_ITEM::LIB_CONVERT::DEMORGAN;
+                return m_bodyStyle == LIB_ITEM::BODY_STYLE::DEMORGAN;
             };
 
     auto multiUnitModeCond =
@@ -523,7 +555,9 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
     mgr->SetConditions( EE_ACTIONS::drawCircle,          EDIT_TOOL( EE_ACTIONS::drawCircle ) );
     mgr->SetConditions( EE_ACTIONS::drawArc,             EDIT_TOOL( EE_ACTIONS::drawArc ) );
     mgr->SetConditions( EE_ACTIONS::drawSymbolLines,     EDIT_TOOL( EE_ACTIONS::drawSymbolLines ) );
+    mgr->SetConditions( EE_ACTIONS::drawSymbolPolygon,   EDIT_TOOL( EE_ACTIONS::drawSymbolPolygon ) );
     mgr->SetConditions( EE_ACTIONS::placeSymbolAnchor,   EDIT_TOOL( EE_ACTIONS::placeSymbolAnchor ) );
+    mgr->SetConditions( EE_ACTIONS::importGraphics,      EDIT_TOOL( EE_ACTIONS::importGraphics ) );
 
 #undef CHECK
 #undef ENABLE
@@ -671,8 +705,8 @@ void SYMBOL_EDIT_FRAME::OnSelectUnit( wxCommandEvent& event )
     if( ( i == wxNOT_FOUND ) || ( ( i + 1 ) == m_unit ) )
         return;
 
-    m_toolManager->RunAction( ACTIONS::cancelInteractive, true );
-    m_toolManager->RunAction( EE_ACTIONS::clearSelection, true );
+    m_toolManager->RunAction( ACTIONS::cancelInteractive );
+    m_toolManager->RunAction( EE_ACTIONS::clearSelection );
 
     m_unit = i + 1;
 
@@ -702,7 +736,7 @@ wxString SYMBOL_EDIT_FRAME::GetCurLib() const
 
     if( !libNickname.empty() )
     {
-        if( !Prj().SchSymbolLibTable()->HasLibrary( libNickname ) )
+        if( !PROJECT_SCH::SchSymbolLibTable( &Prj() )->HasLibrary( libNickname ) )
         {
             Prj().SetRString( PROJECT::SCH_LIBEDIT_CUR_LIB, wxEmptyString );
             libNickname = wxEmptyString;
@@ -717,7 +751,7 @@ wxString SYMBOL_EDIT_FRAME::SetCurLib( const wxString& aLibNickname )
 {
     wxString old = GetCurLib();
 
-    if( aLibNickname.empty() || !Prj().SchSymbolLibTable()->HasLibrary( aLibNickname ) )
+    if( aLibNickname.empty() || !PROJECT_SCH::SchSymbolLibTable( &Prj() )->HasLibrary( aLibNickname ) )
         Prj().SetRString( PROJECT::SCH_LIBEDIT_CUR_LIB, wxEmptyString );
     else
         Prj().SetRString( PROJECT::SCH_LIBEDIT_CUR_LIB, aLibNickname );
@@ -728,7 +762,9 @@ wxString SYMBOL_EDIT_FRAME::SetCurLib( const wxString& aLibNickname )
 
 void SYMBOL_EDIT_FRAME::SetCurSymbol( LIB_SYMBOL* aSymbol, bool aUpdateZoom )
 {
-    m_toolManager->RunAction( EE_ACTIONS::clearSelection, true );
+    wxCHECK( m_toolManager, /* void */ );
+
+    m_toolManager->RunAction( EE_ACTIONS::clearSelection );
     GetCanvas()->GetView()->Clear();
     delete m_symbol;
 
@@ -751,7 +787,7 @@ void SYMBOL_EDIT_FRAME::SetCurSymbol( LIB_SYMBOL* aSymbol, bool aUpdateZoom )
     m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
 
     GetRenderSettings()->m_ShowUnit = m_unit;
-    GetRenderSettings()->m_ShowConvert = m_convert;
+    GetRenderSettings()->m_ShowBodyStyle = m_bodyStyle;
     GetRenderSettings()->m_ShowDisabled = IsSymbolFromLegacyLibrary() && !IsSymbolFromSchematic();
     GetRenderSettings()->m_ShowGraphicsDisabled = IsSymbolAlias() && !IsSymbolFromSchematic();
     GetCanvas()->DisplaySymbol( m_symbol );
@@ -759,7 +795,7 @@ void SYMBOL_EDIT_FRAME::SetCurSymbol( LIB_SYMBOL* aSymbol, bool aUpdateZoom )
     GetCanvas()->GetView()->ClearHiddenFlags();
 
     if( aUpdateZoom )
-        m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+        m_toolManager->RunAction( ACTIONS::zoomFitScreen );
 
     GetCanvas()->Refresh();
 
@@ -794,28 +830,31 @@ void SYMBOL_EDIT_FRAME::SetCurSymbol( LIB_SYMBOL* aSymbol, bool aUpdateZoom )
     }
     else if( IsSymbolAlias() )
     {
-        std::shared_ptr<LIB_SYMBOL> parent = m_symbol->GetParent().lock();
+        wxString rootSymbolName;
 
         // Don't assume the parent symbol shared pointer is still valid.
-        wxCHECK( parent, /* void */ );
+        if( std::shared_ptr<LIB_SYMBOL> rootSymbol = m_symbol->GetRootSymbol() )
+            rootSymbolName = rootSymbol->GetName();
+        else
+        {
+            wxCHECK( false, /* void */ );
+        }
 
-        wxString parentSymbolName = parent->GetName();
         int      unit = GetUnit();
-        int      convert = GetConvert();
+        int      bodyStyle = GetBodyStyle();
         wxString msg;
         wxString link;
 
-        msg.Printf( _( "Symbol %s is derived from %s.  Symbol graphics will not be editable." ),
-                    UnescapeString( symbolName ),
-                    UnescapeString( parentSymbolName ) );
+        msg.Printf( _( "Symbol %s is a derived symbol.  Symbol graphics will not be editable." ),
+                    UnescapeString( symbolName ) );
 
-        link.Printf( _( "Open %s" ), UnescapeString( parentSymbolName ) );
+        link.Printf( _( "Open %s" ), UnescapeString( rootSymbolName ) );
 
         wxHyperlinkCtrl* button = new wxHyperlinkCtrl( infobar, wxID_ANY, link, wxEmptyString );
         button->Bind( wxEVT_COMMAND_HYPERLINK, std::function<void( wxHyperlinkEvent& aEvent )>(
                 [=]( wxHyperlinkEvent& aEvent )
                 {
-                    LoadSymbolFromCurrentLib( parentSymbolName, unit, convert );
+                    LoadSymbolFromCurrentLib( rootSymbolName, unit, bodyStyle );
                 } ) );
 
         infobar->RemoveAllButtons();
@@ -869,8 +908,8 @@ wxString SYMBOL_EDIT_FRAME::AddLibraryFile( bool aCreateNew )
 
     wxFileName fn = m_libMgr->GetUniqueLibraryName();
 
-    if( !LibraryFileBrowser( !aCreateNew, fn, KiCadSymbolLibFileWildcard(),
-                             KiCadSymbolLibFileExtension, false,
+    if( !LibraryFileBrowser( !aCreateNew, fn, FILEEXT::KiCadSymbolLibFileWildcard(),
+                             FILEEXT::KiCadSymbolLibFileExtension, false,
                              ( libTable == &SYMBOL_LIB_TABLE::GetGlobalLibTable() ),
                              PATHS::GetDefaultUserSymbolsPath() ) )
     {
@@ -1110,7 +1149,7 @@ void SYMBOL_EDIT_FRAME::UpdateLibraryTree( const wxDataViewItem& aTreeItem, LIB_
     if( aTreeItem.IsOk() )   // Can be not found in tree if the current footprint is imported
                              // from file therefore not yet in tree.
     {
-        static_cast<LIB_TREE_NODE_LIB_ID*>( aTreeItem.GetID() )->Update( aSymbol );
+        static_cast<LIB_TREE_NODE_ITEM*>( aTreeItem.GetID() )->Update( aSymbol );
         m_treePane->GetLibTree()->RefreshLibTree();
     }
 }
@@ -1163,7 +1202,7 @@ void SYMBOL_EDIT_FRAME::emptyScreen()
     SetCurSymbol( nullptr, false );
     SetScreen( m_dummyScreen );
     ClearUndoRedoList();
-    m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+    m_toolManager->RunAction( ACTIONS::zoomFitScreen );
     Refresh();
 }
 
@@ -1215,6 +1254,8 @@ void SYMBOL_EDIT_FRAME::ShowChangedLanguage()
 
     m_treePane->GetLibTree()->ShowChangedLanguage();
 
+    m_propertiesPanel->LanguageChanged();
+
     // status bar
     UpdateMsgPanel();
 
@@ -1231,19 +1272,27 @@ void SYMBOL_EDIT_FRAME::ShowChangedLanguage()
 void SYMBOL_EDIT_FRAME::SetScreen( BASE_SCREEN* aScreen )
 {
     SCH_BASE_FRAME::SetScreen( aScreen );
+
+    // Let tools add things to the view if necessary
+    if( m_toolManager )
+        m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
 }
 
 
 void SYMBOL_EDIT_FRAME::RebuildView()
 {
     GetRenderSettings()->m_ShowUnit = m_unit;
-    GetRenderSettings()->m_ShowConvert = m_convert;
+    GetRenderSettings()->m_ShowBodyStyle = m_bodyStyle;
     GetRenderSettings()->m_ShowDisabled = IsSymbolFromLegacyLibrary() && !IsSymbolFromSchematic();
     GetRenderSettings()->m_ShowGraphicsDisabled = IsSymbolAlias() && !IsSymbolFromSchematic();
     GetCanvas()->DisplaySymbol( m_symbol );
     GetCanvas()->GetView()->HideDrawingSheet();
     GetCanvas()->GetView()->ClearHiddenFlags();
 
+    // Let tools add things to the view if necessary
+    if( m_toolManager )
+        m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
+
     GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
     GetCanvas()->Refresh();
 }
@@ -1286,7 +1335,55 @@ const BOX2I SYMBOL_EDIT_FRAME::GetDocumentExtents( bool aIncludeAllVisible ) con
     }
     else
     {
-        return m_symbol->Flatten()->GetUnitBoundingBox( m_unit, m_convert );
+        return m_symbol->Flatten()->GetUnitBoundingBox( m_unit, m_bodyStyle );
+    }
+}
+
+
+void SYMBOL_EDIT_FRAME::FocusOnItem( LIB_ITEM* aItem )
+{
+    static KIID lastBrightenedItemID( niluuid );
+
+    LIB_ITEM* lastItem = nullptr;
+
+    if( m_symbol )
+    {
+        for( LIB_PIN* pin : m_symbol->GetAllLibPins() )
+        {
+            if( pin->m_Uuid == lastBrightenedItemID )
+                lastItem = pin;
+        }
+
+        std::vector<LIB_FIELD*> fields;
+
+        m_symbol->GetFields( fields );
+
+        for( LIB_FIELD* field : fields )
+        {
+            if( field->m_Uuid == lastBrightenedItemID )
+                lastItem = field;
+        }
+    }
+
+    if( lastItem && lastItem != aItem )
+    {
+        lastItem->ClearBrightened();
+
+        UpdateItem( lastItem );
+        lastBrightenedItemID = niluuid;
+    }
+
+    if( aItem )
+    {
+        if( !aItem->IsBrightened() )
+        {
+            aItem->SetBrightened();
+
+            UpdateItem( aItem );
+            lastBrightenedItemID = aItem->m_Uuid;
+        }
+
+        FocusOnLocation( VECTOR2I( aItem->GetFocusPosition().x, -aItem->GetFocusPosition().y ) );
     }
 }
 
@@ -1304,7 +1401,7 @@ void SYMBOL_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
             wxString libNickname;
             wxString msg;
 
-            SYMBOL_LIB_TABLE*    libTable = Prj().SchSymbolLibTable();
+            SYMBOL_LIB_TABLE*    libTable = PROJECT_SCH::SchSymbolLibTable( &Prj() );
             const LIB_TABLE_ROW* libTableRow = libTable->FindRowByURI( libFileName );
 
             if( !libTableRow )
@@ -1343,7 +1440,7 @@ void SYMBOL_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
     case MAIL_RELOAD_LIB:
     {
         wxString          currentLib = GetCurLib();
-        SYMBOL_LIB_TABLE* libTable = Prj().SchSymbolLibTable();
+        SYMBOL_LIB_TABLE* libTable = PROJECT_SCH::SchSymbolLibTable( &Prj() );
 
         FreezeLibraryTree();
 
@@ -1361,6 +1458,55 @@ void SYMBOL_EDIT_FRAME::KiwayMailIn( KIWAY_EXPRESS& mail )
         break;
     }
 
+    case MAIL_REFRESH_SYMBOL:
+    {
+        SYMBOL_LIB_TABLE* tbl = PROJECT_SCH::SchSymbolLibTable( &Prj() );
+        LIB_SYMBOL* symbol = GetCurSymbol();
+
+        wxLogTrace( "KICAD_LIB_WATCH", "Received refresh symbol request for %s",
+                    payload );
+
+        if( !tbl || !symbol )
+            break;
+
+        wxString libName = symbol->GetLibId().GetLibNickname();
+        const SYMBOL_LIB_TABLE_ROW* row = tbl->FindRow( libName );
+
+        if( !row )
+            return;
+
+        wxFileName libfullname( row->GetFullURI( true ) );
+
+        wxFileName changedLib( mail.GetPayload() );
+        wxLogTrace( "KICAD_LIB_WATCH",
+                    "Received refresh symbol request for %s, current symbols is %s",
+                    changedLib.GetFullPath(), libfullname.GetFullPath() );
+
+        if( changedLib == libfullname )
+        {
+            wxLogTrace( "KICAD_LIB_WATCH", "Refreshing symbol %s", symbol->GetName() );
+
+            m_libMgr->UpdateLibraryBuffer( libName );
+
+            LIB_SYMBOL* lib_symbol = m_libMgr->GetBufferedSymbol( symbol->GetName(), libName );
+            wxCHECK2_MSG( lib_symbol, break, wxString::Format( "Symbol %s not found in library %s",
+                          symbol->GetName(), libName ) );
+
+            // The buffered screen for the symbol
+            SCH_SCREEN* symbol_screen = m_libMgr->GetScreen( lib_symbol->GetName(), libName );
+
+            SetScreen( symbol_screen );
+            SetCurSymbol( new LIB_SYMBOL( *lib_symbol ), false );
+            RebuildSymbolUnitsList();
+            SetShowDeMorgan( GetCurSymbol()->HasAlternateBodyStyle() );
+
+            if( m_toolManager )
+                m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
+        }
+
+        break;
+    }
+
     default:
         ;
     }
@@ -1410,18 +1556,29 @@ void SYMBOL_EDIT_FRAME::ClearUndoORRedoList( UNDO_REDO_LIST whichList, int aItem
     if( aItemCount == 0 )
         return;
 
-    UNDO_REDO_CONTAINER& list = whichList == UNDO_LIST ? m_undoList : m_redoList;
+    UNDO_REDO_CONTAINER& list = ( whichList == UNDO_LIST ) ? m_undoList : m_redoList;
 
-    for( PICKED_ITEMS_LIST* command : list.m_CommandsList )
+    if( aItemCount < 0 )
     {
-        command->ClearListAndDeleteItems( []( EDA_ITEM* aItem )
-                                          {
-                                              delete aItem;
-                                          } );
-        delete command;
+        list.ClearCommandList();
     }
+    else
+    {
+        for( int ii = 0; ii < aItemCount; ii++ )
+        {
+            if( list.m_CommandsList.size() == 0 )
+                break;
 
-    list.m_CommandsList.clear();
+            PICKED_ITEMS_LIST* curr_cmd = list.m_CommandsList[0];
+            list.m_CommandsList.erase( list.m_CommandsList.begin() );
+
+            curr_cmd->ClearListAndDeleteItems( []( EDA_ITEM* aItem )
+                                               {
+                                                   delete aItem;
+                                               } );
+            delete curr_cmd;    // Delete command
+        }
+    }
 }
 
 
@@ -1481,7 +1638,7 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
             break;
         }
 
-        libField.SetPosition( wxPoint( pos.x, -pos.y ) );
+        libField.SetPosition( VECTOR2I( pos.x, -pos.y ) );
 
         fullSetOfFields.emplace_back( std::move( libField ) );
     }
@@ -1495,7 +1652,7 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
     m_schematicSymbolUUID = aSymbol->m_Uuid;
     m_reference = symbol->GetFieldById( REFERENCE_FIELD )->GetText();
     m_unit = std::max( 1, aSymbol->GetUnit() );
-    m_convert = std::max( 1, aSymbol->GetConvert() );
+    m_bodyStyle = std::max( 1, aSymbol->GetBodyStyle() );
 
     // Optimize default edit options for this symbol
     // Usually if units are locked, graphic items are specific to each unit
@@ -1508,6 +1665,7 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
 
     SetScreen( tmpScreen );
     SetCurSymbol( symbol.release(), true );
+    setSymWatcher( nullptr );
 
     ReCreateMenuBar();
     ReCreateHToolbar();
@@ -1520,9 +1678,15 @@ void SYMBOL_EDIT_FRAME::LoadSymbolFromSchematic( SCH_SYMBOL* aSymbol )
 
     UpdateTitle();
     RebuildSymbolUnitsList();
-    SetShowDeMorgan( GetCurSymbol()->HasConversion() );
+    SetShowDeMorgan( GetCurSymbol()->HasAlternateBodyStyle() );
     UpdateSymbolMsgPanelInfo();
-    Refresh();
+
+    // Let tools add things to the view if necessary
+    if( m_toolManager )
+        m_toolManager->ResetTools( TOOL_BASE::MODEL_RELOAD );
+
+    GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
+    GetCanvas()->Refresh();
 }
 
 
@@ -1532,7 +1696,7 @@ bool SYMBOL_EDIT_FRAME::addLibTableEntry( const wxString& aLibFile, TABLE_SCOPE
     wxFileName libTableFileName( Prj().GetProjectPath(),
                                  SYMBOL_LIB_TABLE::GetSymbolLibTableFileName() );
     wxString libNickname = fn.GetName();
-    SYMBOL_LIB_TABLE* libTable = Prj().SchSymbolLibTable();
+    SYMBOL_LIB_TABLE* libTable = PROJECT_SCH::SchSymbolLibTable( &Prj() );
     const ENV_VAR_MAP& envVars = Pgm().GetLocalEnvVariables();
 
     if( libTable->HasLibrary( libNickname ) )
@@ -1601,7 +1765,7 @@ bool SYMBOL_EDIT_FRAME::replaceLibTableEntry( const wxString& aLibNickname,
     {
         libTableFileName.SetPath( Prj().GetProjectPath() );
         libTableFileName.SetName( SYMBOL_LIB_TABLE::GetSymbolLibTableFileName() );
-        libTable = Prj().SchSymbolLibTable();
+        libTable = PROJECT_SCH::SchSymbolLibTable( &Prj() );
         isGlobalTable = false;
         row = libTable->FindRow( aLibNickname );
     }
