@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2004-2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
+ * Copyright (C) 2023 CERN (www.cern.ch)
  * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
@@ -27,11 +28,11 @@
 
 #include <confirm.h>
 #include <core/arraydim.h>
+#include <core/thread_pool.h>
 #include <gestfich.h>
 #include <pcb_edit_frame.h>
 #include <board_design_settings.h>
 #include <3d_viewer/eda_3d_viewer_frame.h>
-#include <widgets/msgpanel.h>
 #include <fp_lib_table.h>
 #include <kiface_base.h>
 #include <macros.h>
@@ -40,44 +41,44 @@
 #include <wx/snglinst.h>
 #include <netlist_reader/pcb_netlist.h>
 #include <pcbnew_id.h>
-#include <io_mgr.h>
 #include <wildcards_and_files_ext.h>
 #include <tool/tool_manager.h>
 #include <board.h>
-#include <wx/checkbox.h>
-#include <wx/stdpaths.h>
-#include <ratsnest/ratsnest_data.h>
 #include <kiplatform/app.h>
 #include <widgets/appearance_controls.h>
 #include <widgets/wx_infobar.h>
 #include <widgets/wx_progress_reporters.h>
 #include <settings/settings_manager.h>
+#include <string_utf8_map.h>
 #include <paths.h>
 #include <pgm_base.h>
 #include <project/project_file.h>
+#include <project_pcb.h>
 #include <project/project_local_settings.h>
 #include <project/net_settings.h>
-#include <plugins/cadstar/cadstar_pcb_archive_plugin.h>
-#include <plugins/kicad/pcb_plugin.h>
+#include <io/common/plugin_common_choose_project.h>
+#include <pcb_io/pcb_io_mgr.h>
+#include <pcb_io/cadstar/pcb_io_cadstar_archive.h>
+#include <pcb_io/kicad_sexpr/pcb_io_kicad_sexpr.h>
+#include <dialogs/dialog_export_2581.h>
 #include <dialogs/dialog_imported_layers.h>
+#include <dialogs/dialog_import_choose_project.h>
 #include <tools/pcb_actions.h>
 #include "footprint_info_impl.h"
-#include "board_commit.h"
-#include "zone_filler.h"
+#include <board_commit.h>
+#include <zone_filler.h>
 #include <wx_filename.h>  // For ::ResolvePossibleSymlinks()
 
 #include <kiplatform/io.h>
 
-#include <wx/wupdlock.h>
+#include <wx/stdpaths.h>
+#include <wx/ffile.h>
 #include <wx/filedlg.h>
-#include <wx/wfstream.h>
 #include <wx/txtstrm.h>
+#include <wx/wfstream.h>
+#include <wx/zipstrm.h>
 
-#if wxCHECK_VERSION( 3, 1, 7 )
 #include "widgets/filedlg_hook_save_project.h"
-#else
-#include "widgets/legacyfiledlg_save_project.h"
-#endif
 
 //#define     USE_INSTRUMENTATION     1
 #define     USE_INSTRUMENTATION     0
@@ -92,96 +93,67 @@
  * @param aKicadFilesOnly true to list KiCad pcb files plugins only, false to list import plugins.
  * @return  true if chosen, else false if user aborted.
  */
-bool AskLoadBoardFileName( PCB_EDIT_FRAME* aParent, int* aCtl, wxString* aFileName,
-                           bool aKicadFilesOnly )
+bool AskLoadBoardFileName( PCB_EDIT_FRAME* aParent, wxString* aFileName, int aCtl = 0 )
 {
-    // This is a subset of all PLUGINs which are trusted to be able to
-    // load a BOARD. User may occasionally use the wrong plugin to load a
-    // *.brd file (since both legacy and eagle use *.brd extension),
-    // but eventually *.kicad_pcb will be more common than legacy *.brd files.
+    std::vector<IO_BASE::IO_FILE_DESC> descriptions;
 
-    // clang-format off
-    static const struct
+    for( const auto& plugin : PCB_IO_MGR::PLUGIN_REGISTRY::Instance()->AllPlugins() )
     {
-        const wxString&     filter;
-        IO_MGR::PCB_FILE_T  pluginType;
-    } loaders[] =
+        bool isKiCad = plugin.m_type == PCB_IO_MGR::KICAD_SEXP || plugin.m_type == PCB_IO_MGR::LEGACY;
+
+        if( ( aCtl & KICTL_KICAD_ONLY ) && !isKiCad )
+            continue;
+
+        if( ( aCtl & KICTL_NONKICAD_ONLY ) && isKiCad )
+            continue;
+
+        IO_RELEASER<PCB_IO> pi( plugin.m_createFunc() );
+        wxCHECK( pi, false );
+
+        const IO_BASE::IO_FILE_DESC& desc = pi->GetBoardFileDesc();
+
+        if( desc.m_FileExtensions.empty() )
+            continue;
+
+        descriptions.emplace_back( desc );
+    }
+
+    wxString                 fileFiltersStr;
+    std::vector<std::string> allExtensions;
+    std::set<wxString>       allWildcardsSet;
+
+    for( const IO_BASE::IO_FILE_DESC& desc : descriptions )
     {
-        // Current Kicad board files.
-        { PcbFileWildcard(),                    IO_MGR::KICAD_SEXP },
+        if( !fileFiltersStr.IsEmpty() )
+            fileFiltersStr += wxChar( '|' );
 
-        // Old Kicad board files.
-        { LegacyPcbFileWildcard(),              IO_MGR::LEGACY },
+        fileFiltersStr += desc.FileFilter();
 
-        // Import Altium Circuit Maker board files.
-        { AltiumCircuitMakerPcbFileWildcard(),  IO_MGR::ALTIUM_CIRCUIT_MAKER },
-
-        // Import Altium Circuit Studio board files.
-        { AltiumCircuitStudioPcbFileWildcard(), IO_MGR::ALTIUM_CIRCUIT_STUDIO },
-
-        // Import Altium Designer board files.
-        { AltiumDesignerPcbFileWildcard(),      IO_MGR::ALTIUM_DESIGNER },
-
-        // Import Cadstar PCB Archive board files.
-        { CadstarPcbArchiveFileWildcard(),      IO_MGR::CADSTAR_PCB_ARCHIVE },
-
-        // Import Eagle board files.
-        { EaglePcbFileWildcard(),               IO_MGR::EAGLE },
-
-        // Import PCAD board files.
-        { PCadPcbFileWildcard(),                IO_MGR::PCAD },
-
-        // Import Fabmaster board files.
-        { FabmasterPcbFileWildcard(),           IO_MGR::FABMASTER },
-    };
-    // clang-format on
-
-    wxFileName  fileName( *aFileName );
-    wxString    fileFilters;
-
-    if( aKicadFilesOnly )
-    {
-        std::vector<std::string> fileExtensions;
-
-        for( unsigned ii = 0; ii < 2; ++ii )
+        for( const std::string& ext : desc.m_FileExtensions )
         {
-            if( !fileFilters.IsEmpty() )
-                fileFilters += wxChar( '|' );
-
-            fileFilters += wxGetTranslation( loaders[ii].filter );
-
-            PLUGIN::RELEASER plugin( IO_MGR::PluginFind( loaders[ii].pluginType ) );
-            wxCHECK( plugin, false );
-            fileExtensions.push_back( plugin->GetFileExtension().ToStdString() );
+            allExtensions.emplace_back( ext );
+            allWildcardsSet.insert( wxT( "*." ) + formatWildcardExt( ext ) + wxT( ";" ) );
         }
+    }
 
-        fileFilters = _( "All KiCad Board Files" ) + AddFileExtListToFilter( fileExtensions )
-                        + wxT( "|" ) + fileFilters;
+    wxString allWildcardsStr;
+
+    for( const wxString& wildcard : allWildcardsSet )
+        allWildcardsStr << wildcard;
+
+    if( aCtl & KICTL_KICAD_ONLY )
+    {
+        fileFiltersStr = _( "All KiCad Board Files" ) + AddFileExtListToFilter( allExtensions );
     }
     else
     {
-        wxString allWildcards;
-
-        for( unsigned ii = 2; ii < arrayDim( loaders ); ++ii )
-        {
-            if( !fileFilters.IsEmpty() )
-                fileFilters += wxChar( '|' );
-
-            fileFilters += wxGetTranslation( loaders[ii].filter );
-
-            PLUGIN::RELEASER plugin( IO_MGR::PluginFind( loaders[ii].pluginType ) );
-            wxCHECK( plugin, false );
-            allWildcards += wxT( "*." ) + formatWildcardExt( plugin->GetFileExtension() ) +
-                            wxT( ";" );
-        }
-
-        fileFilters = _( "All supported formats" ) + wxT( "|" ) + allWildcards + wxT( "|" ) +
-                      fileFilters;
+        fileFiltersStr = _( "All supported formats" ) + wxT( "|" ) + allWildcardsStr + wxT( "|" )
+                         + fileFiltersStr;
     }
 
-
-    wxString    path;
-    wxString    name;
+    wxFileName fileName( *aFileName );
+    wxString   path;
+    wxString   name;
 
     if( fileName.FileExists() )
     {
@@ -198,15 +170,12 @@ bool AskLoadBoardFileName( PCB_EDIT_FRAME* aParent, int* aCtl, wxString* aFileNa
     }
 
     wxFileDialog dlg( aParent,
-                      aKicadFilesOnly ? _( "Open Board File" ) : _( "Import Non KiCad Board File" ),
-                      path, name, fileFilters, wxFD_OPEN | wxFD_FILE_MUST_EXIST );
+                      ( aCtl & KICTL_KICAD_ONLY ) ? _( "Open Board File" )
+                                                  : _( "Import Non KiCad Board File" ),
+                      path, name, fileFiltersStr, wxFD_OPEN | wxFD_FILE_MUST_EXIST );
 
     if( dlg.ShowModal() == wxID_OK )
     {
-        // For import option, if Eagle (*.brd files), tell OpenProjectFiles() to use Eagle plugin.
-        // It's the only special case because of the duplicate use of the *.brd file extension.
-        // Other cases are clear because of unique file extensions.
-        *aCtl = aKicadFilesOnly ? 0 : KICTL_EAGLE_BRD;
         *aFileName = dlg.GetPath();
         aParent->SetMruPath( wxFileName( dlg.GetPath() ).GetPath() );
         return true;
@@ -229,47 +198,30 @@ bool AskLoadBoardFileName( PCB_EDIT_FRAME* aParent, int* aCtl, wxString* aFileNa
  */
 bool AskSaveBoardFileName( PCB_EDIT_FRAME* aParent, wxString* aFileName, bool* aCreateProject )
 {
-    wxString    wildcard =  PcbFileWildcard();
+    wxString   wildcard = FILEEXT::PcbFileWildcard();
     wxFileName  fn = *aFileName;
 
-    fn.SetExt( KiCadPcbFileExtension );
+    fn.SetExt( FILEEXT::KiCadPcbFileExtension );
 
     wxFileDialog dlg( aParent, _( "Save Board File As" ), fn.GetPath(), fn.GetFullName(), wildcard,
                       wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
 // Add a "Create a project" checkbox in standalone mode and one isn't loaded
-#if wxCHECK_VERSION( 3, 1, 7 )
     FILEDLG_HOOK_SAVE_PROJECT newProjectHook;
 
     if( Kiface().IsSingle() && aParent->Prj().IsNullProject() )
         dlg.SetCustomizeHook( newProjectHook );
-#else
-    if( Kiface().IsSingle() && aParent->Prj().IsNullProject() )
-        dlg.SetExtraControlCreator( &LEGACYFILEDLG_SAVE_PROJECT::Create );
-#endif
-
 
     if( dlg.ShowModal() != wxID_OK )
         return false;
 
-    fn = dlg.GetPath();
+    *aFileName = dlg.GetPath();
+    *aFileName = EnsureFileExtension( *aFileName, FILEEXT::KiCadPcbFileExtension );
 
-    // always enforce filename extension, user may not have entered it.
-    fn.SetExt( KiCadPcbFileExtension );
-
-    *aFileName = fn.GetFullPath();
-
-#if wxCHECK_VERSION( 3, 1, 7 )
     if( newProjectHook.IsAttachedToDialog() )
         *aCreateProject = newProjectHook.GetCreateNewProject();
     else if( !aParent->Prj().IsNullProject() )
         *aCreateProject = true;
-#else
-    if( wxWindow* ec = dlg.GetExtraControl() )
-        *aCreateProject = static_cast<LEGACYFILEDLG_SAVE_PROJECT*>( ec )->GetValue();
-    else if( !aParent->Prj().IsNullProject() )
-        *aCreateProject = true;
-#endif
 
     return true;
 }
@@ -281,15 +233,13 @@ void PCB_EDIT_FRAME::OnFileHistory( wxCommandEvent& event )
 
     if( !!fn )
     {
-        int open_ctl = 0;
-
         if( !wxFileName::IsFileReadable( fn ) )
         {
-            if( !AskLoadBoardFileName( this, &open_ctl, &fn, true ) )
+            if( !AskLoadBoardFileName( this, &fn, KICTL_KICAD_ONLY ) )
                 return;
         }
 
-        OpenProjectFiles( std::vector<wxString>( 1, fn ), open_ctl );
+        OpenProjectFiles( std::vector<wxString>( 1, fn ), KICTL_KICAD_ONLY );
     }
 }
 
@@ -315,20 +265,25 @@ bool PCB_EDIT_FRAME::Files_io_from_id( int id )
     {
     case ID_LOAD_FILE:
     {
-        int         open_ctl = 0;
-        wxString    fileName = Prj().AbsolutePath( GetBoard()->GetFileName() );
+        // Only standalone mode can directly load a new document
+        if( !Kiface().IsSingle() )
+            return false;
 
-        return AskLoadBoardFileName( this, &open_ctl, &fileName, true )
-                   && OpenProjectFiles( std::vector<wxString>( 1, fileName ), open_ctl );
+        int      open_ctl = KICTL_KICAD_ONLY;
+        wxString fileName = Prj().AbsolutePath( GetBoard()->GetFileName() );
+
+        return AskLoadBoardFileName( this, &fileName, open_ctl )
+               && OpenProjectFiles( std::vector<wxString>( 1, fileName ), open_ctl );
     }
 
     case ID_IMPORT_NON_KICAD_BOARD:
     {
-        int         open_ctl = 1;
-        wxString    fileName; // = Prj().AbsolutePath( GetBoard()->GetFileName() );
+        // Note: we explicitly allow this even if not in standalone mode for now, even though it is dangerous.
+        int      open_ctl = KICTL_NONKICAD_ONLY;
+        wxString fileName; // = Prj().AbsolutePath( GetBoard()->GetFileName() );
 
-        return AskLoadBoardFileName( this, &open_ctl, &fileName, false )
-                   && OpenProjectFiles( std::vector<wxString>( 1, fileName ), open_ctl );
+        return AskLoadBoardFileName( this, &fileName, open_ctl )
+               && OpenProjectFiles( std::vector<wxString>( 1, fileName ), open_ctl );
     }
 
     case ID_MENU_RECOVER_BOARD_AUTOSAVE:
@@ -377,11 +332,15 @@ bool PCB_EDIT_FRAME::Files_io_from_id( int id )
 
         ReleaseFile();
 
-        return OpenProjectFiles( std::vector<wxString>( 1, fn.GetFullPath() ) );
+        return OpenProjectFiles( std::vector<wxString>( 1, fn.GetFullPath() ), KICTL_REVERT );
     }
 
     case ID_NEW_BOARD:
     {
+        // Only standalone mode can directly load a new document
+        if( !Kiface().IsSingle() )
+            return false;
+
         if( IsContentModified() )
         {
             wxFileName fileName = GetBoard()->GetFileName();
@@ -456,7 +415,7 @@ bool PCB_EDIT_FRAME::Files_io_from_id( int id )
                 savePath = PATHS::GetDefaultUserProjectsPath();
         }
 
-        wxFileName  fn( savePath.GetPath(), orig_name, KiCadPcbFileExtension );
+        wxFileName  fn( savePath.GetPath(), orig_name, FILEEXT::KiCadPcbFileExtension );
         wxString    filename = fn.GetFullPath();
         bool        createProject = false;
         bool        success = false;
@@ -485,69 +444,6 @@ bool PCB_EDIT_FRAME::Files_io_from_id( int id )
 }
 
 
-// The KIWAY_PLAYER::OpenProjectFiles() API knows nothing about plugins, so
-// determine how to load the BOARD here, with minor assistance from KICTL_EAGLE_BRD
-// bit flag.
-IO_MGR::PCB_FILE_T plugin_type( const wxString& aFileName, int aCtl )
-{
-    IO_MGR::PCB_FILE_T  pluginType;
-
-    wxFileName fn = aFileName;
-
-    // Note: file extensions are expected to be in lower case.
-    // This is not always true, especially when importing files, so the string
-    // comparisons are case insensitive to try to find the suitable plugin.
-
-    if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::LEGACY ) ) == 0 )
-    {
-        wxFileInputStream input( aFileName );
-        bool is_legacy = true;
-
-        if(input.IsOk() && !input.Eof() )
-        {
-            wxTextInputStream text( input );
-            wxString line = text.ReadLine();
-
-            if( !line.StartsWith( wxT( "PCBNEW" ) ) )
-                is_legacy = false;
-        }
-
-        // both legacy and eagle share a common file extension.
-        pluginType = ( aCtl & KICTL_EAGLE_BRD ) || !is_legacy ? IO_MGR::EAGLE : IO_MGR::LEGACY;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::PCAD ) ) == 0 )
-    {
-        pluginType = IO_MGR::PCAD;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::ALTIUM_DESIGNER ) ) == 0 )
-    {
-        pluginType = IO_MGR::ALTIUM_DESIGNER;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::ALTIUM_CIRCUIT_STUDIO ) ) == 0 )
-    {
-        pluginType = IO_MGR::ALTIUM_CIRCUIT_STUDIO;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::ALTIUM_CIRCUIT_MAKER ) ) == 0 )
-    {
-        pluginType = IO_MGR::ALTIUM_CIRCUIT_MAKER;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::CADSTAR_PCB_ARCHIVE ) ) == 0 )
-    {
-        pluginType = IO_MGR::CADSTAR_PCB_ARCHIVE;
-    }
-    else if( fn.GetExt().CmpNoCase( IO_MGR::GetFileExtension( IO_MGR::FABMASTER ) ) == 0 )
-    {
-        pluginType = IO_MGR::FABMASTER;
-    }
-    else
-    {
-        pluginType = IO_MGR::KICAD_SEXP;
-    }
-
-    return pluginType;
-}
-
-
 int PCB_EDIT_FRAME::inferLegacyEdgeClearance( BOARD* aBoard )
 {
     PCB_LAYER_COLLECTOR collector;
@@ -647,7 +543,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     }
 
     wxFileName pro = fullFileName;
-    pro.SetExt( ProjectFileExtension );
+    pro.SetExt( FILEEXT::ProjectFileExtension );
 
     bool is_new = !wxFileName::IsFileReadable( fullFileName );
 
@@ -670,9 +566,15 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     // No save prompt (we already prompted above), and only reset to a new blank board if new
     Clear_Pcb( false, !is_new );
 
-    IO_MGR::PCB_FILE_T  pluginType = plugin_type( fullFileName, aCtl );
+    PCB_IO_MGR::PCB_FILE_T pluginType = PCB_IO_MGR::KICAD_SEXP;
 
-    bool converted =  pluginType != IO_MGR::LEGACY && pluginType != IO_MGR::KICAD_SEXP;
+    if( !is_new )
+        pluginType = PCB_IO_MGR::FindPluginTypeFromBoardPath( fullFileName, aCtl );
+
+    if( pluginType == PCB_IO_MGR::FILE_TYPE_NONE )
+        return false;
+
+    bool converted =  pluginType != PCB_IO_MGR::LEGACY && pluginType != PCB_IO_MGR::KICAD_SEXP;
 
     // Loading a project should only be done under carefully considered circumstances.
 
@@ -712,35 +614,54 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     }
     else
     {
-        BOARD*           loadedBoard = nullptr;   // it will be set to non-NULL if loaded OK
-        PLUGIN::RELEASER pi( IO_MGR::PluginFind( pluginType ) );
+        BOARD*              loadedBoard = nullptr;   // it will be set to non-NULL if loaded OK
+        IO_RELEASER<PCB_IO> pi( PCB_IO_MGR::PluginFind( pluginType ) );
 
-        LAYER_REMAPPABLE_PLUGIN* layerRemappablePlugin =
-            dynamic_cast< LAYER_REMAPPABLE_PLUGIN* >( (PLUGIN*) pi );
+        LAYER_REMAPPABLE_PLUGIN* layerRemappableIO = dynamic_cast<LAYER_REMAPPABLE_PLUGIN*>( pi.get() );
 
-        if( layerRemappablePlugin )
+        if( layerRemappableIO )
         {
-            layerRemappablePlugin->RegisterLayerMappingCallback(
+            layerRemappableIO->RegisterLayerMappingCallback(
                     std::bind( DIALOG_IMPORTED_LAYERS::GetMapModal, this, std::placeholders::_1 ) );
         }
 
-        // This will rename the file if there is an autosave and the user want to recover
-		CheckForAutoSaveFile( fullFileName );
+        PROJECT_CHOOSER_PLUGIN* projectChooserIO = dynamic_cast<PROJECT_CHOOSER_PLUGIN*>( pi.get() );
+
+        if( projectChooserIO )
+        {
+            projectChooserIO->RegisterChooseProjectCallback(
+                    std::bind( DIALOG_IMPORT_CHOOSE_PROJECT::GetSelectionsModal, this,
+                               std::placeholders::_1 ) );
+        }
+
+        if( ( aCtl & KICTL_REVERT ) )
+        {
+            DeleteAutoSaveFile( fullFileName );
+        }
+        else
+        {
+            // This will rename the file if there is an autosave and the user wants to recover
+            CheckForAutoSaveFile( fullFileName );
+        }
 
         bool failedLoad = false;
 
         try
         {
+            if( pi == nullptr )
+            {
+                // There was no plugin found, e.g. due to invalid file extension, file header,...
+                THROW_IO_ERROR( _( "File format is not supported" ) );
+            }
+
             STRING_UTF8_MAP props;
-            char        xbuf[30];
-            char        ybuf[30];
 
-            // EAGLE_PLUGIN can use this info to center the BOARD, but it does not yet.
-            snprintf( xbuf, sizeof(xbuf), "%d", GetPageSizeIU().x );
-            snprintf( ybuf, sizeof(ybuf), "%d", GetPageSizeIU().y );
+            if( m_importProperties )
+                props.insert( m_importProperties->begin(), m_importProperties->end() );
 
-            props["page_width"]  = xbuf;
-            props["page_height"] = ybuf;
+            // PCB_IO_EAGLE can use this info to center the BOARD, but it does not yet.
+            props["page_width"] = std::to_string( GetPageSizeIU().x );
+            props["page_height"] = std::to_string( GetPageSizeIU().y );
 
             pi->SetQueryUserCallback(
                     [&]( wxString aTitle, int aIcon, wxString aMessage, wxString aAction ) -> bool
@@ -760,11 +681,12 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
             unsigned startTime = GetRunningMicroSecs();
 #endif
 
-            loadedBoard = pi->Load( fullFileName, nullptr, &props, &Prj(), &progressReporter );
+            pi->SetProgressReporter( &progressReporter );
+            loadedBoard = pi->LoadBoard( fullFileName, nullptr, &props, &Prj() );
 
 #if USE_INSTRUMENTATION
             unsigned stopTime = GetRunningMicroSecs();
-            printf( "PLUGIN::Load(): %u usecs\n", stopTime - startTime );
+            printf( "PCB_IO::Load(): %u usecs\n", stopTime - startTime );
 #endif
         }
         catch( const FUTURE_FORMAT_ERROR& ffe )
@@ -795,7 +717,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
             failedLoad = true;
         }
 
-        if( failedLoad )
+        if( failedLoad || !loadedBoard )
         {
             // We didn't create a new blank board above, so do that now
             Clear_Pcb( false );
@@ -844,18 +766,18 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
             loadedBoard->SetModified();
         }
 
-        // we should not ask PLUGINs to do these items:
+        // we should not ask PCB_IOs to do these items:
         loadedBoard->BuildListOfNets();
         ResolveDRCExclusions( true );
-        m_toolManager->RunAction( PCB_ACTIONS::repairBoard, true, true);
+        m_toolManager->RunAction( PCB_ACTIONS::repairBoard, true);
 
         if( loadedBoard->IsModified() )
             OnModify();
         else
             GetScreen()->SetContentModified( false );
 
-        if( ( pluginType == IO_MGR::LEGACY )
-         || ( pluginType == IO_MGR::KICAD_SEXP
+        if( ( pluginType == PCB_IO_MGR::LEGACY )
+         || ( pluginType == PCB_IO_MGR::KICAD_SEXP
                 && loadedBoard->GetFileFormatVersionAtLoad() < SEXPR_BOARD_FILE_VERSION
                 && loadedBoard->GetGenerator().Lower() != wxT( "gerbview" ) ) )
         {
@@ -886,7 +808,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
             // which prompts the user to continue with overwrite or abort)
             if( newLibPath.Length() > 0 )
             {
-                PLUGIN::RELEASER piSexpr( IO_MGR::PluginFind( IO_MGR::KICAD_SEXP ) );
+                IO_RELEASER<PCB_IO> piSexpr( PCB_IO_MGR::PluginFind( PCB_IO_MGR::KICAD_SEXP ) );
 
                 for( FOOTPRINT* footprint : loadedFootprints )
                 {
@@ -908,7 +830,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
                     }
                 }
 
-                FP_LIB_TABLE*   prjlibtable = Prj().PcbFootprintLibs();
+                FP_LIB_TABLE*   prjlibtable = PROJECT_PCB::PcbFootprintLibs( &Prj() );
                 const wxString& project_env = PROJECT_VAR_NAME;
                 wxString        rel_path, env_path;
 
@@ -928,7 +850,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
                 try
                 {
-                    Prj().PcbFootprintLibs()->Save( tblName );
+                    PROJECT_PCB::PcbFootprintLibs( &Prj() )->Save( tblName );
                 }
                 catch( const IO_ERROR& ioe )
                 {
@@ -956,7 +878,7 @@ bool PCB_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
         wxFileName fn = fullFileName;
 
         if( converted )
-            fn.SetExt( PcbFileExtension );
+            fn.SetExt( FILEEXT::PcbFileExtension );
 
         wxString fname = fn.GetFullPath();
 
@@ -1016,8 +938,8 @@ bool PCB_EDIT_FRAME::SavePcbFile( const wxString& aFileName, bool addToHistory,
     // please, keep it simple.  prompting goes elsewhere.
     wxFileName pcbFileName = aFileName;
 
-    if( pcbFileName.GetExt() == LegacyPcbFileExtension )
-        pcbFileName.SetExt( KiCadPcbFileExtension );
+    if( pcbFileName.GetExt() == FILEEXT::LegacyPcbFileExtension )
+        pcbFileName.SetExt( FILEEXT::KiCadPcbFileExtension );
 
     // Write through symlinks, don't replace them
     WX_FILENAME::ResolvePossibleSymlinks( pcbFileName );
@@ -1036,8 +958,8 @@ bool PCB_EDIT_FRAME::SavePcbFile( const wxString& aFileName, bool addToHistory,
     wxFileName rulesFile( pcbFileName );
     wxString   msg;
 
-    projectFile.SetExt( ProjectFileExtension );
-    rulesFile.SetExt( DesignRulesFileExtension );
+    projectFile.SetExt( FILEEXT::ProjectFileExtension );
+    rulesFile.SetExt( FILEEXT::DesignRulesFileExtension );
 
     if( projectFile.FileExists() )
     {
@@ -1076,9 +998,9 @@ bool PCB_EDIT_FRAME::SavePcbFile( const wxString& aFileName, bool addToHistory,
 
     try
     {
-        PLUGIN::RELEASER    pi( IO_MGR::PluginFind( IO_MGR::KICAD_SEXP ) );
+        IO_RELEASER<PCB_IO> pi( PCB_IO_MGR::PluginFind( PCB_IO_MGR::KICAD_SEXP ) );
 
-        pi->Save( tempFile, GetBoard(), nullptr );
+        pi->SaveBoard( tempFile, GetBoard(), nullptr );
     }
     catch( const IO_ERROR& ioe )
     {
@@ -1148,7 +1070,7 @@ bool PCB_EDIT_FRAME::SavePcbFile( const wxString& aFileName, bool addToHistory,
     if( m_infoBar->GetMessageType() == WX_INFOBAR::MESSAGE_TYPE::OUTDATED_SAVE )
         m_infoBar->Dismiss();
 
-    if( m_infoBar->IsShown() && m_infoBar->HasCloseButton() )
+    if( m_infoBar->IsShownOnScreen() && m_infoBar->HasCloseButton() )
         m_infoBar->Dismiss();
 
     GetScreen()->SetContentModified( false );
@@ -1159,10 +1081,7 @@ bool PCB_EDIT_FRAME::SavePcbFile( const wxString& aFileName, bool addToHistory,
 
 bool PCB_EDIT_FRAME::SavePcbCopy( const wxString& aFileName, bool aCreateProject )
 {
-    wxFileName  pcbFileName = aFileName;
-
-    // Ensure the file ext is the right ext:
-    pcbFileName.SetExt( KiCadPcbFileExtension );
+    wxFileName pcbFileName( EnsureFileExtension( aFileName, FILEEXT::KiCadPcbFileExtension ) );
 
     if( !IsWritable( pcbFileName ) )
     {
@@ -1179,11 +1098,11 @@ bool PCB_EDIT_FRAME::SavePcbCopy( const wxString& aFileName, bool aCreateProject
 
     try
     {
-        PLUGIN::RELEASER    pi( IO_MGR::PluginFind( IO_MGR::KICAD_SEXP ) );
+        IO_RELEASER<PCB_IO> pi( PCB_IO_MGR::PluginFind( PCB_IO_MGR::KICAD_SEXP ) );
 
         wxASSERT( pcbFileName.IsAbsolute() );
 
-        pi->Save( pcbFileName.GetFullPath(), GetBoard(), nullptr );
+        pi->SaveBoard( pcbFileName.GetFullPath(), GetBoard(), nullptr );
     }
     catch( const IO_ERROR& ioe )
     {
@@ -1198,8 +1117,8 @@ bool PCB_EDIT_FRAME::SavePcbCopy( const wxString& aFileName, bool aCreateProject
     wxFileName rulesFile( pcbFileName );
     wxString   msg;
 
-    projectFile.SetExt( ProjectFileExtension );
-    rulesFile.SetExt( DesignRulesFileExtension );
+    projectFile.SetExt( FILEEXT::ProjectFileExtension );
+    rulesFile.SetExt( FILEEXT::DesignRulesFileExtension );
 
     if( aCreateProject && !projectFile.FileExists() )
         GetSettingsManager()->SaveProjectCopy( projectFile.GetFullPath() );
@@ -1235,7 +1154,7 @@ bool PCB_EDIT_FRAME::doAutoSave()
     if( GetBoard()->GetFileName().IsEmpty() )
     {
         tmpFileName = wxFileName( PATHS::GetDefaultUserProjectsPath(), NAMELESS_PROJECT,
-                                  KiCadPcbFileExtension );
+                                  FILEEXT::KiCadPcbFileExtension );
         GetBoard()->SetFileName( tmpFileName.GetFullPath() );
     }
     else
@@ -1291,19 +1210,156 @@ bool PCB_EDIT_FRAME::doAutoSave()
 }
 
 
-bool PCB_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
+bool PCB_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType,
+                                 const STRING_UTF8_MAP* aProperties )
 {
-    switch( (IO_MGR::PCB_FILE_T) aFileType )
-    {
-    case IO_MGR::CADSTAR_PCB_ARCHIVE:
-    case IO_MGR::EAGLE:
-        return OpenProjectFiles( std::vector<wxString>( 1, aFileName ),
-                                 KICTL_EAGLE_BRD | KICTL_IMPORT_LIB );
+    m_importProperties = aProperties;
 
-    default:
-        return false;
+    switch( (PCB_IO_MGR::PCB_FILE_T) aFileType )
+    {
+    case PCB_IO_MGR::CADSTAR_PCB_ARCHIVE:
+    case PCB_IO_MGR::EAGLE:
+    case PCB_IO_MGR::EASYEDA:
+    case PCB_IO_MGR::EASYEDAPRO:
+        return OpenProjectFiles( std::vector<wxString>( 1, aFileName ),
+                                 KICTL_NONKICAD_ONLY | KICTL_IMPORT_LIB );
+
+    default: break;
     }
 
+    m_importProperties = nullptr;
+
     return false;
 }
 
+
+void PCB_EDIT_FRAME::GenIPC2581File( wxCommandEvent& event )
+{
+    DIALOG_EXPORT_2581 dlg( this );
+
+    if( dlg.ShowModal() != wxID_OK )
+        return;
+
+    wxFileName pcbFileName = dlg.GetOutputPath();
+
+    // Write through symlinks, don't replace them
+    WX_FILENAME::ResolvePossibleSymlinks( pcbFileName );
+
+    if( pcbFileName.GetName().empty() )
+    {
+        DisplayError( this, _( "The board must be saved before generating IPC2581 file." ) );
+        return;
+    }
+
+    if( !IsWritable( pcbFileName ) )
+    {
+        wxString msg = wxString::Format( _( "Insufficient permissions to write file '%s'." ),
+                                         pcbFileName.GetFullPath() );
+
+        DisplayError( this, msg );
+        return;
+    }
+
+    wxString   tempFile = wxFileName::CreateTempFileName( wxS( "pcbnew_ipc" ) );
+    wxString   upperTxt;
+    wxString   lowerTxt;
+    WX_PROGRESS_REPORTER reporter( this, _( "Generating IPC2581 file" ), 5 );
+    STRING_UTF8_MAP props;
+
+    props["units"] = dlg.GetUnitsString();
+    props["sigfig"] = dlg.GetPrecision();
+    props["version"] = dlg.GetVersion();
+    props["OEMRef"] = dlg.GetOEM();
+    props["mpn"] = dlg.GetMPN();
+    props["mfg"] = dlg.GetMfg();
+    props["dist"] = dlg.GetDist();
+    props["distpn"] = dlg.GetDistPN();
+
+    auto saveFile = [&]() -> bool
+    {
+        try
+        {
+            IO_RELEASER<PCB_IO> pi( PCB_IO_MGR::PluginFind( PCB_IO_MGR::IPC2581 ) );
+            pi->SetProgressReporter( &reporter );
+            pi->SaveBoard( tempFile, GetBoard(), &props );
+            return true;
+        }
+        catch( const IO_ERROR& ioe )
+        {
+            DisplayError( this, wxString::Format( _( "Error generating IPC2581 file '%s'.\n%s" ),
+                                                  pcbFileName.GetFullPath(), ioe.What() ) );
+
+            lowerTxt.Printf( _( "Failed to create temporary file '%s'." ), tempFile );
+
+            SetMsgPanel( upperTxt, lowerTxt );
+
+            // In case we started a file but didn't fully write it, clean up
+            wxRemoveFile( tempFile );
+
+            return false;
+        }
+    };
+
+    thread_pool& tp = GetKiCadThreadPool();
+    auto ret = tp.submit( saveFile );
+
+
+    std::future_status status = ret.wait_for( std::chrono::milliseconds( 250 ) );
+
+    while( status != std::future_status::ready )
+    {
+        reporter.KeepRefreshing();
+        status = ret.wait_for( std::chrono::milliseconds( 250 ) );
+    }
+
+    try
+    {
+        if( !ret.get() )
+            return;
+    }
+    catch(const std::exception& e)
+    {
+        wxLogError( "Exception in IPC2581 generation: %s", e.what() );
+        GetScreen()->SetContentModified( false );
+        return;
+    }
+
+    // Preserve the permissions of the current file
+    KIPLATFORM::IO::DuplicatePermissions( pcbFileName.GetFullPath(), tempFile );
+
+    if( dlg.GetCompress() )
+    {
+        wxFileName tempfn = pcbFileName;
+        tempfn.SetExt( FILEEXT::Ipc2581FileExtension );
+        wxFileName zipfn = tempFile;
+        zipfn.SetExt( "zip" );
+
+        wxFFileOutputStream fnout( zipfn.GetFullPath() );
+        wxZipOutputStream zip( fnout );
+        wxFFileInputStream fnin( tempFile );
+
+        zip.PutNextEntry( tempfn.GetFullName() );
+        fnin.Read( zip );
+        zip.Close();
+        fnout.Close();
+
+        wxRemoveFile( tempFile );
+        tempFile = zipfn.GetFullPath();
+    }
+
+    // If save succeeded, replace the original with what we just wrote
+    if( !wxRenameFile( tempFile, pcbFileName.GetFullPath() ) )
+    {
+        DisplayError( this, wxString::Format( _( "Error generating IPC2581 file '%s'.\n"
+                                                 "Failed to rename temporary file '%s." ),
+                                              pcbFileName.GetFullPath(),
+                                              tempFile ) );
+
+        lowerTxt.Printf( _( "Failed to rename temporary file '%s'." ),
+                         tempFile );
+
+        SetMsgPanel( upperTxt, lowerTxt );
+    }
+
+    GetScreen()->SetContentModified( false );
+}
